@@ -6,26 +6,28 @@ _get_ves()
 {
 	local cmd=$1
 	local rm_empty='sed s/^-[[:space:]]*//'
+	# Only allow this completion when user has root privileges
+	[ "$EUID" -ne 0 ] && exit 1
 	case $cmd in
 		create)
-			# create a new VEID, by increasing the last one
-			local veids=`/usr/sbin/vzlist -H -a -oveid | tail -1`
+			# create a new CTID, by increasing the last one
+			local veids=`/usr/sbin/vzlist -H -a -octid | tail -1`
 			[ -n "$veids" ] || veids=100
 			echo $((veids+1))
 			;;
-		start|mount|umount|destroy)
-			# stopped VEs
-			/usr/sbin/vzlist -H -S -oveid
+		start|mount|umount|destroy|delete)
+			# stopped CTs
+			/usr/sbin/vzlist -H -S -octid
 			/usr/sbin/vzlist -H -S -oname | $rm_empty
 			;;
 		stop|enter|exec*)
-			# running VEs
-			/usr/sbin/vzlist -H -oveid
-			/usr/sbin/vzlist -H -oname | $rm_emtpy
+			# running CTs
+			/usr/sbin/vzlist -H -octid
+			/usr/sbin/vzlist -H -oname | $rm_empty
 			;;
 		*)
-			# All VEs
-			/usr/sbin/vzlist -H -a -oveid
+			# All CTs
+			/usr/sbin/vzlist -H -a -octid
 			/usr/sbin/vzlist -H -a -oname | $rm_empty
 			;;
 	esac
@@ -40,16 +42,16 @@ _vzctl()
 	#echo "COMP_WORDS: $COMP_WORDS"
 	#echo "COMP_CWORD: $COMP_CWORD"
 	#echo "COMP_WORDS[1]: ${COMP_WORDS[1]}"
-	
+
 	COMPREPLY=()
 	cur=${COMP_WORDS[COMP_CWORD]}
 	prev=${COMP_WORDS[COMP_CWORD-1]}
 
 	vzctl_common_opts="--quiet --verbose --help --version"
-	vzctl_cmds="create destroy mount umount chkpnt restore \
+	vzctl_cmds="create destroy delete mount umount chkpnt restore \
 		set start stop restart status enter exec exec2 runscript"
 	vzctl_create_opts="--ostemplate --config --private --root \
-		--ipadd --hostname FIXME"
+		--ipadd --hostname --name --description"
 	vzctl_set_opts="--save --onboot --root --private \
 		--userpasswd --disabled --ipadd --ipdel \
 		--hostname --nameserver --searchdomain \
@@ -57,11 +59,15 @@ _vzctl()
 		--kmemsize --tcpsndbuf --tcprcvbuf --othersockbuf \
 		--dgramrcvbuf --oomguarpages --lockedpages --privvmpages \
 		--shmpages --numfile --numflock --numpty --numsiginfo \
-		--dcachesize --numiptent --physpages --cpuunits --cpulimit \
+		--dcachesize --numiptent --physpages --swappages \
+		--cpuunits --cpulimit --cpus \
 		--iptables --netdev_add --netdev_del --netif_add --netif_del \
 		--diskspace --diskinodes --quotatime --quotaugidlimit \
 		--noatime --capability --devnodes --devices --applyconfig \
-		--name --ioprio --setmode --meminfo"
+		--name --ioprio --setmode --meminfo --bootorder --features"
+	vzctl_start_opts="--wait --force"
+	vzctl_stop_opts="--fast"
+	vzctl_restart_opts="$vzctl_start_opts $vzctl_stop_opts"
 
 	iptables_names="iptable_filter iptable_mangle ipt_limit
 		ipt_multiport ipt_tos ipt_TOS ipt_REJECT ipt_TCPMSS
@@ -75,8 +81,9 @@ _vzctl()
 		sys_module sys_rawio sys_chroot sys_ptrace sys_pacct
 		sys_admin sys_boot sys_nice sys_resource sys_time
 		sys_tty_config mknod lease setveid ve_admin"
-		
-									  
+
+	feature_names="sysfs nfs sit ipip ppp ipgre bridge nfsd"
+
 
 	case $COMP_CWORD in
 	1)
@@ -94,7 +101,7 @@ _vzctl()
 			COMPREPLY=( $( compgen -W "$vzctl_cmds" -- $cur ) )
 			;;
 		*)
-			# VEID
+			# CTID
 			COMPREPLY=( $( compgen -W "$(_get_ves $prev)" -- $cur ) )
 			;;
 		esac
@@ -102,16 +109,16 @@ _vzctl()
 
 	*) # COMP_CWORD >= 3
 		if [[ $COMP_CWORD -eq 3 && ! $prev != '^[1-9][0-9]*$' ]]; then
-			# VEID
+			# CTID
 			COMPREPLY=( $( compgen -W "$(_get_ves $prev)" -- $cur ) )
 		else
-	
+
 			# flag or option
 			case $prev in
 			--ostemplate)
 				# get the template path from the vz.conf
 				local vztmpl=`grep ^TEMPLATE /etc/vz/vz.conf | cut -d "=" -f 2`
-				COMPREPLY=( $( compgen -W "$(ls -1 $vztmpl/cache/*.tar.gz | \
+				COMPREPLY=( $( compgen -W "$(command ls -1 $vztmpl/cache/*.tar.gz 2>/dev/null |
 						    sed -e "s#^$vztmpl/cache/##" -e 's#.tar.gz$##')" -- $cur ) )
 
 				;;
@@ -122,7 +129,7 @@ _vzctl()
 				COMPREPLY=( $( compgen -W "restart ignore" -- $cur ) )
 				;;
 			--config|--applyconfig)
-				local configs=$(ls -1 /etc/vz/conf/*.conf-sample | \
+				local configs=$(command ls -1 /etc/vz/conf/*.conf-sample |
 						    cut -d "-" -f 2- | sed -e 's#.conf-sample$##')
 
 				configs=${configs/.conf-sample/}
@@ -161,6 +168,11 @@ _vzctl()
 #					"${cap}:on ${cap}:off" -- $cur) )
 #				fi
 				;;
+			--features)
+				# featurename:on|off
+				COMPREPLY=( $( compgen -W "$feature_names" -- $cur ) )
+				# FIXME: add :on or :off -- can't make it work
+				;;
 			--devnodes)
 				# FIXME: device:r|w|rw|none
 				local devs=''
@@ -172,13 +184,16 @@ _vzctl()
 				COMPREPLY=( $( compgen -W "$devices" -- $cur ) )
 				;;
 			--ipdel)
-				# Get VEID
+				# Only allow this completion
+				# when user has root privileges
+				[ "$EUID" -ne 0 ] && exit 1
+				# Get CTID
 				local ve=${COMP_WORDS[2]}
 				if [[ ! ${ve} != '^[1-9][0-9]*$' ]] ; then
 					# --verbose or --quiet used
 					ve=${COMP_WORDS[3]}
 				fi
-				# VENAME or VEID ?
+				# NAME or ID ?
 				LIST_OPT=`echo $ve | awk '/[a-zA-Z]/ {print "-N"}'`
 				local ips="`/usr/sbin/vzlist -H -o ip $LIST_OPT $ve | grep -vi -`"
 				COMPREPLY=( $( compgen -W "$ips all" -- $cur ) )
@@ -191,9 +206,9 @@ _vzctl()
 						# --verbose or --quiet used
 						ve=${COMP_WORDS[3]}
 					fi
-					# get veth(ernet) device of VE, proc and config
+					# get veth(ernet) device of CT, proc and config
 					local netif_proc="`awk "/${ve}/ { print \\$4 }" /proc/vz/veth`"
-					local netif_conf="`grep ^NETIF /etc/vz/conf/${ve}.conf | \
+					local netif_conf="`grep ^NETIF /etc/vz/conf/${ve}.conf |
 						sed -e 's/"/\n/g' -e 's/;/\n/g' -e 's/,/\n/g' -e 's/=/ /g' | awk '/^ifname/ {print \$2}'`"
 					netif="$netif_proc $netif_conf"
 				fi
@@ -215,7 +230,7 @@ _vzctl()
 						# --verbose or --quiet used
 						cmd=${COMP_WORDS[2]}
 					fi
-		
+
 					case "$cmd" in
 					create)
 						COMPREPLY=( $( compgen -W "$vzctl_create_opts" -- $cur ) )
@@ -226,8 +241,14 @@ _vzctl()
 					chkpnt|restore)
 						COMPREPLY=( $( compgen -W "--dumpfile" -- $cur ) )
 						;;
+					start)
+						COMPREPLY=( $( compgen -W "$vzctl_start_opts" -- $cur ) )
+						;;
 					stop)
-						COMPREPLY=( $( compgen -W "--fast" -- $cur ) )
+						COMPREPLY=( $( compgen -W "$vzctl_stop_opts" -- $cur ) )
+						;;
+					restart)
+						COMPREPLY=( $( compgen -W "$vzctl_restart_opts" -- $cur ) )
 						;;
 					*)
 						;;
